#
# $Id: play_post.t,v 1.1.1.1 2003/09/23 09:47:26 mshiltonj Exp $
#
#  Copyright (c) 2001, Raphael Manfredi
#  
#  You may redistribute only under the terms of the Artistic License,
#  as specified in the README file that comes with the distribution.
#
# HISTORY
# $Log: play_post.t,v $
# Revision 1.1.1.1  2003/09/23 09:47:26  mshiltonj
# Initial Import
#
# Revision 0.1  2001/03/31 10:54:04  ram
# Baseline for first Alpha release.
#
# $EndLog$
#

require "t/browse.pl";
browse(-method => 'POST');

